@@ -65,9 +65,20 @@ extern "C"
 
 using namespace amf;
 
+#ifdef __clang__
+    #pragma clang diagnostic push
+    #pragma clang diagnostic ignored "-Wexit-time-destructors"
+    #pragma clang diagnostic ignored "-Wglobal-constructors"
+#endif
+
 static const amf_string AMF_FORBIDDEN_SYMBOLS = ":? %,;@&=+$<>#\"";
 static const amf_string AMF_FORBIDDEN_SYMBOLS_QUERY = ":? %,;@+$<>#\"";
 
+#ifdef __clang__
+    #pragma clang diagnostic pop
+#endif
+
+
 //MM: in question:  unwise      = "{" | "}" | "|" | "\" | "^" | "[" | "]" | "`"
 
 //----------------------------------------------------------------------------------------
@@ -562,7 +573,7 @@ amf_string AMF_STD_CALL amf::amf_string_format(const char* format, ...)
 //----------------------------------------------------------------------------------------
 amf_wstring AMF_STD_CALL amf::amf_string_formatVA(const wchar_t* format, va_list args)
 {
-#if defined(__linux) || defined(__APPLE__)
+#if (defined(__linux) || defined(__APPLE__)) && (!defined(__ANDROID__))
     //replace %s with %ls
     amf_wstring text(format);
     amf_wstring textReplaced;
@@ -575,6 +586,10 @@ amf_wstring AMF_STD_CALL amf::amf_string_formatVA(const wchar_t* format, va_list
             textReplaced.push_back(L'l');
             textReplaced.push_back(L's');
         }
+        else if(percentFlag && (*i == L'S'))
+        {
+            textReplaced.push_back(L's');
+        }
         else
         {
             textReplaced.push_back(*i);
