@@ -198,9 +198,6 @@ namespace amf
         return amf_release_semaphore(m_hSemaphore, 1, &iOldCount);
     }
     //----------------------------------------------------------------------------
-
-    //----------------------------------------------------------------------------
-    //----------------------------------------------------------------------------
     AMFLock::AMFLock(AMFSyncBase* pBase, amf_ulong ulTimeout)
         : m_pBase(pBase),
         m_bLocked()
@@ -347,8 +344,9 @@ namespace amf
         AMFThread*      m_pOwner;
         uintptr_t       m_pThread;
         AMFEvent        m_StopEvent;
+        AMFCriticalSection m_Lock;
     public:
-        // this icalled by owner
+        // this is called by owner
         AMFThreadObj(AMFThread* owner);
         virtual ~AMFThreadObj();
 
@@ -377,9 +375,10 @@ namespace amf
         }
     };
     //----------------------------------------------------------------------------
-    AMFThreadObj::AMFThreadObj(AMFThread* owner)
-        : m_pThread(uintptr_t(-1)),
-        m_StopEvent(true, true), m_pOwner(owner)
+    AMFThreadObj::AMFThreadObj(AMFThread* owner) :
+        m_pOwner(owner),
+        m_pThread(uintptr_t(-1)),
+        m_StopEvent(true, true)
     {}
     //----------------------------------------------------------------------------
     AMFThreadObj::~AMFThreadObj()
@@ -411,7 +410,7 @@ namespace amf
         {
             return true;
         }
-
+        AMFLock lock(&m_Lock);
         m_pThread = _beginthread(AMFThreadProc, 0, (void* )this);
 
         return m_pThread != (uintptr_t)-1L;
@@ -431,11 +430,15 @@ namespace amf
     //----------------------------------------------------------------------------
     bool AMFThreadObj::WaitForStop()
     {
+        AMFLock lock(&m_Lock);
         if(m_pThread == (uintptr_t)-1L)
         {
             return true;
         }
-        return m_StopEvent.Lock();
+        bool stopped = m_StopEvent.Lock();
+        
+        m_pThread = (uintptr_t)-1L;
+        return stopped;
     }
     //----------------------------------------------------------------------------
     bool AMFThreadObj::StopRequested()
@@ -447,7 +450,7 @@ namespace amf
         return m_pThread != (uintptr_t)-1L;
     }
     //----------------------------------------------------------------------------
-    void amf::ExitThread()
+    void ExitThread()
     {
         _endthread();
     }
@@ -475,7 +478,8 @@ namespace amf
         AMFThread*      m_pOwner;
         pthread_t m_hThread;
         bool m_bStopRequested;
-        pthread_mutex_t m_hMutex;
+        bool m_bRunning;
+        AMFCriticalSection m_Lock;
 
         AMFThreadObj(const AMFThreadObj&);
         AMFThreadObj& operator=(const AMFThreadObj&);
@@ -485,16 +489,15 @@ namespace amf
 
     AMFThreadObj::AMFThreadObj(AMFThread* owner)
         : m_pOwner(owner),
-        m_hThread(0),
         m_bStopRequested(false),
-        m_hMutex()
+        m_bRunning(false)
     {
-        pthread_mutex_init(&m_hMutex, 0);
     }
 
     AMFThreadObj::~AMFThreadObj()
     {
-        pthread_mutex_destroy(&m_hMutex);
+        RequestStop();
+        WaitForStop();
     }
 
     void* AMF_CDECL_CALL AMFThreadObj::AMFThreadProc(void* pThis)
@@ -504,9 +507,9 @@ namespace amf
         {
             return 0;
         }
+
         pT->Run();
         pT->Terminate();
-        pT->m_hThread = 0;
         pT->m_bStopRequested = false;
         return 0;
     }
@@ -516,60 +519,55 @@ namespace amf
         bool result = true;
         if (IsRunning() == false)
         {
-            result = (0 == pthread_create(&m_hThread, 0, AMFThreadProc, (void*)this));
+            WaitForStop();
+            
+            AMFLock lock(&m_Lock);
+            if (pthread_create(&m_hThread, 0, AMFThreadProc, (void*)this) == 0)
+            {
+                m_bRunning = true;
+            }
+            else
+            {
+                result = false;
+            }
         }
         return result;
     }
 
     bool AMFThreadObj::RequestStop()
     {
-    #if defined(__APPLE__)
-        if(m_hThread == nullptr)
-    #else
-        if(m_hThread == (uintptr_t)0L)
-    #endif
+        AMFLock lock(&m_Lock);
+        if (IsRunning() == false)
         {
             return true;
         }
       
-        pthread_mutex_lock(&m_hMutex);
         m_bStopRequested = true;
-        pthread_mutex_unlock(&m_hMutex);
         return true;
     }
 
     bool AMFThreadObj::WaitForStop()
     {
-    #if defined(__APPLE__)
-        if(m_hThread != nullptr)
-    #else
-        if(m_hThread != (uintptr_t)0L)
-    #endif
+        AMFLock lock(&m_Lock);
+
+        if (IsRunning() == true)
         {
             pthread_join(m_hThread, 0);
-            m_hThread = 0;
+            m_bRunning = false;
         }
             
         m_bStopRequested = false;
-
         return true;
     }
 
     bool AMFThreadObj::StopRequested()
     {
-        pthread_mutex_lock(&m_hMutex);
-        bool bRet = m_bStopRequested;
-        pthread_mutex_unlock(&m_hMutex);
-        return bRet;
+        return m_bStopRequested;
     }
 
     bool AMFThreadObj::IsRunning()
     {
-    #if defined(__APPLE__)
-        return m_hThread != nullptr;
-    #else
-        return m_hThread != (uintptr_t)0L;
-    #endif
+        return m_bRunning;
     }
 
     void ExitThread()
