@@ -90,6 +90,8 @@ extern "C"
     void        AMF_CDECL_CALL amf_restore_timer_precision();
 
     amf_handle  AMF_CDECL_CALL amf_load_library(const wchar_t* filename);
+    amf_handle  AMF_CDECL_CALL amf_load_library1(const wchar_t* filename, bool bGlobal);
+
     void*       AMF_CDECL_CALL amf_get_proc_address(amf_handle module, const char* procName);
     int         AMF_CDECL_CALL amf_free_library(amf_handle module);
 
@@ -543,7 +545,7 @@ namespace amf
                 {
                     bStop = true;
                 }
-#if defined(__linux)
+#if defined(__linux) || defined(__APPLE__)
                 ///< HACK
                 ///< This amf_sleep(0) is required to emulate windows mutex behavior.
                 ///< In Windows release mutext causes some other waiting thread is receiving ownership of mutex.
