@@ -63,8 +63,8 @@ static const wchar_t* pCodec = AMFVideoEncoderVCE_AVC;
 //static const wchar_t *pCodec = AMFVideoEncoder_HEVC;
 static const bool isH264 = (amf_wstring(pCodec) == amf_wstring(AMFVideoEncoderVCE_AVC)) ? true : false;
 
-//static const amf_int picMgmtMode = (isH264) ? AMF_VIDEO_ENCODER_PICTURE_TRANSFER_MODE_OFF : AMF_VIDEO_ENCODER_HEVC_PICTURE_TRANSFER_MODE_OFF; // disable ref/recon pic transfer
-static const amf_int picMgmtMode = (isH264) ? AMF_VIDEO_ENCODER_PICTURE_TRANSFER_MODE_ON : AMF_VIDEO_ENCODER_HEVC_PICTURE_TRANSFER_MODE_ON;     // enable ref/recon pic transfer
+//static const amf_int64 picMgmtMode = (isH264) ? AMF_VIDEO_ENCODER_PICTURE_TRANSFER_MODE_OFF : AMF_VIDEO_ENCODER_HEVC_PICTURE_TRANSFER_MODE_OFF; // disable ref/recon pic transfer
+static const amf_int64 picMgmtMode = (isH264) ? amf_int64(AMF_VIDEO_ENCODER_PICTURE_TRANSFER_MODE_ON) : amf_int64(AMF_VIDEO_ENCODER_HEVC_PICTURE_TRANSFER_MODE_ON);     // enable ref/recon pic transfer
 
 static const bool injectReconDump = true; // inject the reconstructed picture dump from the encoder
 //static const bool injectReconDump = false; // inject the frame generated by this sample
@@ -222,8 +222,8 @@ int main(int argc, char* argv[])
 
     if(isH264)
     { 
-        res = encoder->SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCONDING);
-        AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCONDING) failed");
+        res = encoder->SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCODING);
+        AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCODING) failed");
 
         if(bMaximumSpeed)
         {
@@ -254,8 +254,8 @@ int main(int argc, char* argv[])
     }
     else
     {
-        res = encoder->SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCONDING);
-        AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCONDING)");
+        res = encoder->SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCODING);
+        AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCODING)");
 
         if(bMaximumSpeed)
         {
@@ -362,10 +362,14 @@ int main(int argc, char* argv[])
         }
 
         res = encoder->SubmitInput(surfaceIn);
-        if(res == AMF_INPUT_FULL) // handle full queue
+        if(res == AMF_NEED_MORE_INPUT) // handle full queue
         {
-            amf_sleep(1); // input queue is full: wait, poll and submit again
+            // do nothing
         }
+		else if (res == AMF_INPUT_FULL || res == AMF_DECODER_NO_FREE_SURFACES)
+		{
+			amf_sleep(1); // input queue is full: wait, poll and submit again
+		}
         else
         {
             AMF_RETURN_IF_FAILED(res, L"SubmitInput() failed");
@@ -858,6 +862,11 @@ void PollingThread::Run()
         {
             break; // Drain complete
         }
+		if ((res != AMF_OK) && (res != AMF_REPEAT))
+		{
+			// trace possible error message
+			break; // Drain complete
+		}
         if(data != NULL)
         {
             amf_pts poll_time = amf_high_precision_clock();
