@@ -65,7 +65,7 @@ static amf_int32 heightIn					= 0;
 static amf_int32 frameRateIn				= 0;
 static amf_int32 submitted					= 0;
 
-static wchar_t *pOutputCodec;
+static const wchar_t *pOutputCodec;
 static amf_int64 bitRateOut					= 5000000L; // in bits, 5MBit
 static bool bMaximumSpeed					= true;
 
@@ -227,8 +227,8 @@ int main(int argc, char* argv[])
 	if (amf_wstring(pOutputCodec) == amf_wstring(AMFVideoEncoderVCE_AVC))
 	{
 		swprintf(fileNameEncOutWithSize, amf_countof(fileNameEncOutWithSize), fileNameEncOut, parser->GetPictureWidth(), parser->GetPictureHeight(), 264);
-		res = encoder->SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCONDING);
-		AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCONDING) failed");
+		res = encoder->SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCODING);
+		AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCODING) failed");
 
 		if (bMaximumSpeed)
 		{
@@ -259,8 +259,8 @@ int main(int argc, char* argv[])
 	else
 	{
 		swprintf(fileNameEncOutWithSize, amf_countof(fileNameEncOutWithSize), fileNameEncOut, parser->GetPictureWidth(), parser->GetPictureHeight(), 265);
-		res = encoder->SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCONDING);
-		AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCONDING)");
+		res = encoder->SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCODING);
+		AMF_RETURN_IF_FAILED(res, L"SetProperty(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCODING)");
 
 		if (bMaximumSpeed)
 		{
@@ -462,6 +462,11 @@ void DecPollingThread::Run()
         {
             break; // Drain complete
         }
+		if ((res != AMF_OK) && (res != AMF_REPEAT))
+		{
+			// trace possible error message
+			break; // Drain complete
+		}
         if(data != NULL)
         {
             WaitDecoder(m_pContext, amf::AMFSurfacePtr(data)); // Waits till decoder finishes decode the surface. Need for accurate profiling only. Do not use in the product!!!
@@ -567,6 +572,11 @@ void EncPollingThread::Run()
 		{
 			break; // Drain complete
 		}
+		if ((res != AMF_OK) && (res != AMF_REPEAT))
+		{
+			// trace possible error message
+			break; // Drain complete
+		}
 		if (data != NULL)
 		{
 			amf::AMFBufferPtr buffer(data); // query for buffer interface
