@@ -447,7 +447,7 @@ StitchPipelineBase::StitchPipelineBase() :
 
     //SetParam(PARAM_NAME_ENCODER_CODEC, AMFVideoEncoderVCE_AVC);
 
-    m_EncoderParamsAVC.SetParam(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCONDING);
+    m_EncoderParamsAVC.SetParam(AMF_VIDEO_ENCODER_USAGE, AMF_VIDEO_ENCODER_USAGE_TRANSCODING);
     m_EncoderParamsAVC.SetParam(AMF_VIDEO_ENCODER_QUALITY_PRESET, AMF_VIDEO_ENCODER_QUALITY_PRESET_SPEED);
     m_EncoderParamsAVC.SetParam(AMF_VIDEO_ENCODER_TARGET_BITRATE, bitrate);
     m_EncoderParamsAVC.SetParam(AMF_VIDEO_ENCODER_PEAK_BITRATE, bitrate * 3 /2);
@@ -458,7 +458,7 @@ StitchPipelineBase::StitchPipelineBase() :
     m_EncoderParamsAVC.SetParam(AMF_VIDEO_ENCODER_B_PIC_PATTERN, 0);
     m_EncoderParamsAVC.SetParam(AMF_VIDEO_ENCODER_RATE_CONTROL_METHOD, AMF_VIDEO_ENCODER_RATE_CONTROL_METHOD_PEAK_CONSTRAINED_VBR);
 
-    m_EncoderParamsHEVC.SetParam(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCONDING);
+    m_EncoderParamsHEVC.SetParam(AMF_VIDEO_ENCODER_HEVC_USAGE, AMF_VIDEO_ENCODER_HEVC_USAGE_TRANSCODING);
     m_EncoderParamsHEVC.SetParam(AMF_VIDEO_ENCODER_HEVC_QUALITY_PRESET, AMF_VIDEO_ENCODER_HEVC_QUALITY_PRESET_SPEED);
     m_EncoderParamsHEVC.SetParam(AMF_VIDEO_ENCODER_HEVC_TARGET_BITRATE, bitrate);
     m_EncoderParamsHEVC.SetParam(AMF_VIDEO_ENCODER_HEVC_PEAK_BITRATE, bitrate * 3 / 2);
